//
//  UMTransportSession.h
//  ulibtransport
//
//  Created by Andreas Fink on 07.06.18.
//  Copyright © 2018 Andreas Fink. All rights reserved.
//


typedef enum UMTransportDialogState
{
    UMTransportDialogState_closed = 0,
    UMTransportDialogState_opening = 1,
    UMTransportDialogState_open = 2,
    UMTransportDialogState_closing = 3,
} UMTransportDialogState;

#import <ulibtcap/ulibtcap.h>

#import "UMTransportErrorCode.h"
@class UMTransportService;
@class UMTransportSegment;
@class UMTransportMessage;
@class UMTransportOpen;
@class UMTransportClose;

@interface UMTransportDialog : UMObject
{
    SccpAddress *_localAddress;
    SccpAddress *_remoteAddress;
    UMTCAP_UserDialogIdentifier *_dialogIdentifier; /* the reference tcap uses to talk to us */
    NSString *_userDialogRef; /* the reference given to us by  a user for a new dialog */
    UMTransportService  *_transportService;
    NSString *_localTransactionId;
    NSString *_remoteTransactionId;
    UMLayerTCAP *_tcap;
    UMTCAP_Variant _variant;
    UMTCAP_asn1_dialoguePortion *_dialoguePortion;
    NSDictionary *_incomingOptions;
    UMMutex *_lock;
    NSMutableDictionary *_waitingSegments;
    NSTimeInterval  _segmentTimeout;
    BOOL _sessionIsClosing;
    BOOL _sessionIsClosed;
    BOOL _transportAllowed;
    NSDate *_lastActive;
    int64_t _nextInvokeId;
    UMMutex *_segmentsLock;
    NSUInteger _maxSegmentSize;
    NSMutableDictionary<NSNumber *, NSMutableDictionary<NSNumber *,id> *> *_pendingSegments; /* a dictionary who's keys is the invokeID and who's content is a dictionary who's key is the segment sequence number and the content is the payload data segments except in the case of the sequence number being @(-1) where the payload is a NSValue of the total count or -1 */
    
    UMTransportDialogState _dialogState;
}

@property(readwrite,strong) SccpAddress *localAddress;
@property(readwrite,strong) SccpAddress *remoteAddress;
@property(readwrite,strong) UMTCAP_UserDialogIdentifier *dialogIdentifier;
@property(readwrite,strong) UMTransportService  *transportService;
@property(readwrite,strong) NSString *localTransactionId;
@property(readwrite,strong) NSString *remoteTransactionId;
@property(readwrite,strong) UMLayerTCAP *tcap;
@property(readwrite,assign) UMTCAP_Variant variant;
@property(readwrite,strong) UMTCAP_asn1_dialoguePortion *dialoguePortion;
@property(readwrite,strong) NSDictionary *incomingOptions;
@property(readwrite,assign) NSTimeInterval segmentTimeout;
@property(readwrite,strong) NSString *userDialogRef;
@property(readwrite,assign) BOOL sessionIsClosing;
@property(readwrite,assign) BOOL sessionIsClosed;
@property(readwrite,assign) NSUInteger maxSegmentSize;
@property(readwrite,assign) UMTransportDialogState dialogState;


- (int64_t)nextInvokeId;

- (int64_t)sendOpen:(UMTransportOpen *)packet;
- (void)sendOpenAcceptForInvokeId:(int64_t)invokeId;

- (void)sendOpenRejectForInvokeId:(int64_t)invokeId
                           reason:(int)reason
                       reasonText:(NSString *)reasonText;

- (int64_t)sendClose:(UMTransportClose *)packet;
- (void)sendCloseAcceptForInvokeId:(int64_t)invokeId;



- (void) processIncomingOpenForInvoke:(int64_t)invokeId
                               params:(UMTransportOpen *)openParams;

- (void) processIncomingCloseForInvoke:(int64_t)invokeId
                                params:(UMTransportClose *)closeParams;

- (void) processIncomingSegmentForInvoke:(int64_t)invokeId
                                  params:(UMTransportSegment *)segment
                                    last:(BOOL)last;

- (void)processIncomingInvoke:(UMASN1Object *)params
                       opCode:(int64_t)opcode
                     invokeId:(int64_t)invokeId
                         last:(BOOL)isLast
                      options:(NSDictionary *)options;


- (void)processIncomingComponents:(TCAP_NSARRAY_OF_COMPONENT_PDU *)components
                          options:(NSDictionary *)options
                          willEnd:(BOOL)willEnd;
- (void)processIncomingMessage:(UMTransportMessage *)msg;


-(void) processIncomingPAbort:(NSDictionary *)options
                 callingAddress:(SccpAddress *)src
                  calledAddress:(SccpAddress *)dst
                dialoguePortion:(UMTCAP_asn1_dialoguePortion *)xdialoguePortion
                  transactionId:(NSString *)localTransactionId
          remoteTransactionId:(NSString *)remoteTransactionId;

-(void)processIncomingUAbort:(NSDictionary *)options
                 callingAddress:(SccpAddress *)src
                  calledAddress:(SccpAddress *)dst
                dialoguePortion:(UMTCAP_asn1_dialoguePortion *)xdialoguePortion
                  transactionId:(NSString *)localTransactionId
         remoteTransactionId:(NSString *)remoteTransactionId;


- (void) executeIncomingOpen:(UMTransportMessage *)msg invokeId:(int64_t)invokeId;
- (void) executeIncomingClose:(UMTransportMessage *)msg invokeId:(int64_t)invokeId;
- (void) executeIncomingMessage:(UMTransportMessage *)message invokeId:(int64_t)invokeId;
- (void) sendOutgoingMessage:(UMTransportMessage *)message invokeId:(int64_t)invokeId;
- (void) sendOutgoingError:(UMTransportErrorCode)errorCode invokeId:(int64_t)invokeId;

@end
