//
//  UMTransportService.h
//  ulibtransport
//
//  Created by Andreas Fink on 07.06.18.
//  Copyright © 2018 Andreas Fink. All rights reserved.
//

#import <ulibtcap/ulibtcap.h>
#import "UMTransportProviderProtocol.h"
#import "UMTransportUserProtocol.h"

@class UMTransportDialog;
@class SccpAddress;
@class UMGSMMAP_AddressString;
@class UMTransportPacket;

@interface UMTransportService : UMLayer<UMTCAP_UserProtocol,UMTransportProviderProtocol>
{
    SccpAddress         *_localAddress;
    UMMutex             *_lock;
    int                 _lastRef;
    NSMutableDictionary *_currentDialogs;
    NSMutableDictionary *_dialogForDestination;

    NSTimeInterval      _segmentTimeout;
    NSString            *_instanceName;
    UMLayerTCAP         *_tcap;
    id<UMTransportUserProtocol> _delegate;
    NSMutableArray<UMTransportPacket *>  *_waitingPackets;
}

@property(readwrite,strong) SccpAddress     *localAddress;
@property(readwrite,assign) NSTimeInterval  segmentTimeout;
@property(readwrite,strong) NSString  *instanceName;
@property(readwrite,strong) UMTCAP_TransactionIdPool *transactionIdPool;
@property(readwrite,strong) UMLayerTCAP *tcap;
@property(readwrite,strong) id<UMTransportUserProtocol> delegate;

- (UMTransportService *)initWithTaskQueueMulti:(UMTaskQueueMulti *)tq name:(NSString *)name;

- (void)tcapBeginIndication:(UMTCAP_UserDialogIdentifier *)userDialogId
          tcapTransactionId:(NSString *)localTransactionId
    tcapRemoteTransactionId:(NSString *)remoteTransactionId
                    variant:(UMTCAP_Variant)var
             callingAddress:(SccpAddress *)src
              calledAddress:(SccpAddress *)dst
            dialoguePortion:(UMTCAP_asn1_dialoguePortion *)xdialoguePortion
               callingLayer:(UMLayerTCAP *)tcapLayer
                 components:(TCAP_NSARRAY_OF_COMPONENT_PDU *)components
                    options:(NSDictionary *)options;

- (void)tcapContinueIndication:(UMTCAP_UserDialogIdentifier *)userDialogId
             tcapTransactionId:(NSString *)localTransactionId
       tcapRemoteTransactionId:(NSString *)remoteTransactionId
                       variant:(UMTCAP_Variant)var
                callingAddress:(SccpAddress *)src
                 calledAddress:(SccpAddress *)dst
               dialoguePortion:(UMTCAP_asn1_dialoguePortion *)xdialoguePortion
                  callingLayer:(UMLayerTCAP *)tcapLayer
                    components:(TCAP_NSARRAY_OF_COMPONENT_PDU *)components
                       options:(NSDictionary *)options;

- (void)tcapEndIndication:(UMTCAP_UserDialogIdentifier *)userDialogId
        tcapTransactionId:(NSString *)localTransactionId
  tcapRemoteTransactionId:(NSString *)remoteTransactionId
                  variant:(UMTCAP_Variant)var
           callingAddress:(SccpAddress *)src
            calledAddress:(SccpAddress *)dst
          dialoguePortion:(UMTCAP_asn1_dialoguePortion *)xdialoguePortion
             callingLayer:(UMLayerTCAP *)tcapLayer
               components:(TCAP_NSARRAY_OF_COMPONENT_PDU *)components
                  options:(NSDictionary *)options;


- (void)tcapUnidirectionalIndication:(UMTCAP_UserDialogIdentifier *)userDialogId
                   tcapTransactionId:(NSString *)localTransactionId
             tcapRemoteTransactionId:(NSString *)remoteTransactionId
                             variant:(UMTCAP_Variant)variant
                      callingAddress:(SccpAddress *)src
                       calledAddress:(SccpAddress *)dst
                     dialoguePortion:(UMTCAP_asn1_dialoguePortion *)xdialoguePortion
                        callingLayer:(UMLayerTCAP *)tcapLayer
                          components:(TCAP_NSARRAY_OF_COMPONENT_PDU *)components
                             options:(NSDictionary *)options;


- (void)tcapNoticeIndication:(UMTCAP_UserDialogIdentifier *)userDialogId
           tcapTransactionId:(NSString *)localTransactionId
     tcapRemoteTransactionId:(NSString *)remoteTransactionId
                     variant:(UMTCAP_Variant)variant
              callingAddress:(SccpAddress *)src
               calledAddress:(SccpAddress *)dst
             dialoguePortion:(UMTCAP_asn1_dialoguePortion *)xdialoguePortion
                callingLayer:(UMLayerTCAP *)tcapLayer
                  components:(TCAP_NSARRAY_OF_COMPONENT_PDU *)components
                      reason:(SCCP_ReturnCause)reason
                     options:(NSDictionary *)options;



- (void)tcapPAbortIndication:(UMTCAP_UserDialogIdentifier *)userDialogId
           tcapTransactionId:(NSString *)localTransactionId
     tcapRemoteTransactionId:(NSString *)remoteTransactionId
                     variant:(UMTCAP_Variant)variant
              callingAddress:(SccpAddress *)src
               calledAddress:(SccpAddress *)dst
             dialoguePortion:(UMTCAP_asn1_dialoguePortion *)xdialoguePortion
                callingLayer:(UMLayerTCAP *)tcapLayer
                        asn1:(UMASN1Object *)asn1
                     options:(NSDictionary *)options;


- (void)tcapUAbortIndication:(UMTCAP_UserDialogIdentifier *)userDialogId
           tcapTransactionId:(NSString *)localTransactionId
     tcapRemoteTransactionId:(NSString *)remoteTransactionId
                     variant:(UMTCAP_Variant)variant
              callingAddress:(SccpAddress *)src
               calledAddress:(SccpAddress *)dst
             dialoguePortion:(UMTCAP_asn1_dialoguePortion *)xdialoguePortion
                callingLayer:(UMLayerTCAP *)tcapLayer
                        asn1:(UMASN1Object *)asn1
                     options:(NSDictionary *)options;


- (UMASN1Object *)decodeASN1:(UMASN1Object *)params
               operationCode:(int64_t)opcode
               operationType:(UMTCAP_InternalOperation)operation
               operationName:(NSString **)xoperationName
                     context:(id)context;

- (NSString *)decodeError:(int)err;

- (UMTransportDialog *)dialogById:(UMTCAP_UserDialogIdentifier *)tcapUserId;

/*- (void)handleUMTPdu:(NSData *)pdu
     fromSccpAddress:(SccpAddress *)src
       toSccpAddress:(SccpAddress *)dst;
*/
- (void)handleUMTPduSCCP:(NSData *)pdu
                  source:(SccpAddress *)src
             destination:(SccpAddress *)dst;

- (void)handleUMTSMS:(NSData *)data
              source:(NSString *)src
         destination:(NSString *)dst;

- (void)executeMessage:(UMTransportMessage *)m;
- (void)sendMessage:(UMTransportMessage *)m;

@end
